import argparse
import sys
import pandas as pd
from eapdataset import Dataset
from pathlib import Path


def main(args):
    parser = argparse.ArgumentParser(description='Write result to csv file')
    parser.add_argument("--sur_wtg_dts", type=str, required=True)
    parser.add_argument("--dwntm_recds_dts", type=str, required=True)
    parser.add_argument("--wtg_10m_dts", type=str, required=True)
    parser.add_argument("--resultfile", type=str, required=True)

    args = parser.parse_args(args)


    
    #获取数据
    data_location = Dataset.get_by_name(args.sur_wtg_dts).to_pandas_dataframe(use_cached_result=False)
    data_stop = Dataset.get_by_name(args.dwntm_recds_dts).to_pandas_dataframe(use_cached_result=False)
    data_10min = Dataset.get_by_name(args.wtg_10m_dts).to_pandas_dataframe(use_cached_result=False)

    data_10min = data_10min.dropna()#去空值
    data_10min = data_10min.drop_duplicates()#去重复行
    
    #判断周围风机1有重合的故障时间段
    def wtg_around_cover_time(data, start_time, stop_time):
        stop_count = 0
        #如果周围风机有故障数据
        if len(data) > 0:
            for k in range(len(data)):
                #如果周围风机停机时间、恢复时间在计算风机起始时间内
                if start_time < data['开始时间'][k] <stop_time:
                    stop_count = stop_count + 1
                if start_time < data['结束时间'][k] <stop_time:
                    stop_count = stop_count + 1
                if (start_time > data['开始时间'][k]) and (data['结束时间'][k] > stop_time):
                    stop_count = stop_count + 1
        return stop_count
    #*****************************************************************************


    #类型转换
    def strtofloat(var1):
        if isinstance(var1,float):
            result=var1
        elif isinstance(var1,int):
            result=float(var1)
        elif isinstance(var1,str):
            list1=var1.split('.',3)

            if len(list1)>1:
                var1=list1[0]+'.'+list1[1]
            else:
                var1=list1[0]

            if len(var1.strip()) == 0:
                result=0
            else:
                result=float(var1)
        else:
            result=0
        return result



    #计算周围风机电量
    def calculate_electricity(around_wtg,start_time,stop_time):

        if len(str(around_wtg)) >0:
            data_select = data_10min[(data_10min['风机编号'] == around_wtg) & (data_10min['genactivepw'].astype(float) > 0)]

            #筛选功率大于0的值
            data_select = data_select[data_select['data_time'] >= start_time]
            #data_select = data_select[data_select['时间'] <= stop_time].reset_index(drop=True)
            data_select = data_select[data_select['data_time'] <= stop_time].reset_index(drop=True)

            if len(data_select) == 1:
                time_delta = ((stop_time - start_time).total_seconds())/3600.
                electricity_lost = round(float(data_select['genactivepw'][0])* (time_delta),2)

            if len(data_select) == 0:
                electricity_lost = 0
            if len(data_select)>1:
                #time_delta_top = ((data_select['时间'][0] - start_time).total_seconds())/3600.
                time_delta_top = ((data_select['data_time'][0] - start_time).total_seconds())/3600.
                time_delta_top=strtofloat(time_delta_top)
                #time_delta_bottom = ((stop_time - data_select['时间'][len(data_select) - 1]).total_seconds())/3600.
                time_delta_bottom = ((stop_time - data_select['data_time'][len(data_select) - 1]).total_seconds())/3600.
                #print(start_time,stop_time,time_delta_top)
                #如果平均功率列表中有空行，间隔大于10min则舍弃此行
                if time_delta_top>(1/6.):
                    time_delta_top = 0
                if time_delta_bottom>(1/6.):
                    time_delta_bottom = 0
                #electricity_lost_top = (data_select['值'][0]) * time_delta_top


                electricity_lost_top = (strtofloat(data_select['genactivepw'][0])) * time_delta_top
                #electricity_lost_bottom = (data_select['值'][len(data_select) - 1]) * time_delta_bottom
                electricity_lost_bottom = strtofloat(data_select['genactivepw'][len(data_select) - 1]) * time_delta_bottom
                #electricity_lost_middle = (data_select['值'].sum() - data_select['值'][len(data_select) - 1])/6.

                electricity_lost_middle = (strtofloat(data_select['genactivepw'].sum()) - strtofloat(data_select['genactivepw'][len(data_select) - 1]))/6.
                #icity_lost_middle = (data_select['genactivepw'].sum().typeas(float) - data_select['genactivepw'][len(data_select) - 1].typeas(float))/6.

                electricity_lost = round(strtofloat(electricity_lost_top) + strtofloat(electricity_lost_bottom) + strtofloat(electricity_lost_middle),2)
        else:
            electricity_lost = 0
        return electricity_lost
    #*****************************************************************************



    #风机名称字符串处理，取后两位字符串转换为int
    data_10min['风机编号'] = data_10min['device_id']
    data_stop['风机'] = data_stop['device_id']
    data_stop['状态描述'] = data_stop['iec_level4_description']
    data_stop['开始时间'] = data_stop['start_time']
    data_stop['结束时间'] = data_stop['end_time']

    #*****************************************************************************
    #停机记录表新增列
    data_stop['周围可用风机1'] = ''
    data_stop['周围可用风机2'] = ''
    data_stop['电量损失1（kWh）'] = ''
    data_stop['电量损失2（kWh）'] = ''
    data_stop['电量损失均值（kWh）'] = ''


    #筛选故障停机
    data_stop = data_stop[~(data_stop['状态描述']=='正常发电')].reset_index(drop=True)
    #*****************************************************************************


    #筛选周围风机并进行计算
    #for i in range(len(data_stop)):
    for i in range(0,2000):    

        #获得本条记录风机编号
        wtg_id = data_stop['风机'][i]

        #如果停机时间不是空值，恢复时间是空值，计算昨日电量
        if pd.notna(data_stop['开始时间'][i])==True and pd.notna(data_stop['结束时间'][i])==False:
            time_delta_today = pd.to_datetime('today').date()-(data_stop['开始时间'][i]).date()
            #如果停机时间在昨日
            if time_delta_today ==pd.Timedelta('1 day'):
                data_stop.loc[i,'结束时间'] = pd.to_datetime((data_stop['停机时间'][i]).strftime('%Y/%m/%d')) + pd.Timedelta('1 day')
            if time_delta_today >pd.Timedelta('1 day'):
                #如果停机时间在昨日前
                data_stop.loc[i,'开始时间'] = pd.to_datetime((pd.to_datetime('today')).strftime('%Y/%m/%d')) - pd.Timedelta('1 day')
                data_stop.loc[i,'结束时间'] = pd.to_datetime((pd.to_datetime('today')).strftime('%Y/%m/%d'))



        #如果停机时间、恢复时间均不是空值，计算起始时间段内的电量
        if pd.notna(data_stop['开始时间'][i])==True and pd.notna(data_stop['结束时间'][i])==True:
            #本条记录停机时间及恢复时间
            time_start = data_stop['开始时间'][i]
            time_stop = data_stop['结束时间'][i]

            #找到周围风机及对应距离
            data_find = data_location[data_location['风机运行编号'] == wtg_id]

            if not data_find.empty:
                data_find_index=data_find.index[0]


                #周围风机编号
                key_one = data_find['周围风机1'][data_find_index]
                key_two = data_find['周围风机2'][data_find_index]
                key_three = data_find['周围风机3'][data_find_index]
                key_four = data_find['周围风机4'][data_find_index]
                key_five = data_find['周围风机5'][data_find_index]
                #周围风机对应距离
                value_one = data_find['周围风机1距离（km)'][data_find_index]
                value_two = data_find['周围风机2距离（km)'][data_find_index]
                value_three = data_find['周围风机3距离（km)'][data_find_index]
                value_four = data_find['周围风机4距离（km)'][data_find_index]       
                value_five = data_find['周围风机5距离（km)'][data_find_index]   

                #新建字典类型变量，存放周围风机及对应距离
                wtg_dict = {key_one:value_one, key_two:value_two, key_three:value_three, key_four:value_four, key_five:value_five}
                #判断周围风机是否停机
                #筛选周围风机数据并重新排序
                data_one = (data_stop[data_stop['风机'] == key_one]).reset_index(drop=True)
                data_two = data_stop[data_stop['风机'] == key_two].reset_index(drop=True)
                data_three = data_stop[data_stop['风机'] == key_three].reset_index(drop=True)
                data_four = data_stop[data_stop['风机'] == key_four].reset_index(drop=True)
                data_five = data_stop[data_stop['风机'] == key_five].reset_index(drop=True)

                #判断周围风机有重合的故障时间段，有重合的故障时间段，此台风机不可用
                if wtg_around_cover_time(data_one,time_start,time_stop) >=1:
                    wtg_dict.pop(key_one)
                if wtg_around_cover_time(data_two,time_start,time_stop) >=1:
                    wtg_dict.pop(key_two)
                if wtg_around_cover_time(data_three,time_start,time_stop) >=1:
                    wtg_dict.pop(key_three)
                if wtg_around_cover_time(data_four,time_start,time_stop) >=1:
                    wtg_dict.pop(key_four)
                if wtg_around_cover_time(data_five,time_start,time_stop) >=1:
                    wtg_dict.pop(key_five)


                #选择计算用风机号
                if len(wtg_dict) >= 2:
                    #可用周围风机1，取字典变量中值最小的
                    wtg_target_one = (min(wtg_dict.items(),key=lambda x:x[1]))[0]
                    data_stop.loc[i,'周围可用风机1'] = wtg_target_one
                    #删除此最小值
                    wtg_dict.pop(wtg_target_one)
                    #再次选择最小值
                    wtg_target_two = (min(wtg_dict.items(),key=lambda x:x[1]))[0]
                    data_stop.loc[i,'周围可用风机2'] = wtg_target_two

                if len(wtg_dict) == 1:
                    wtg_target = (min(wtg_dict.items(),key=lambda x:x[1]))[0]
                    data_stop.loc[i,'周围可用风机1'] = wtg_target
                    data_stop.loc[i,'周围可用风机2'] = 0
                if len(wtg_dict) == 0:
                    data_stop.loc[i,'周围可用风机1'] = 0
                    data_stop.loc[i,'周围可用风机2'] = 0
                #电量计算
                #print(data_stop['周围可用风机1'][i])
                data_stop.loc[i,'电量损失1（kWh）'] = calculate_electricity(data_stop['周围可用风机1'][i],time_start,time_stop)        
                data_stop.loc[i,'电量损失2（kWh）'] = calculate_electricity(data_stop['周围可用风机2'][i],time_start,time_stop)
                #判断周围风机计算值是否有空值
                if data_stop['电量损失1（kWh）'][i] == 0 and data_stop['电量损失2（kWh）'][i] > 0:
                    data_stop.loc[i,'电量损失均值（kWh）'] = data_stop['电量损失2（kWh）'][i]
                if data_stop['电量损失2（kWh）'][i] == 0 and data_stop['电量损失1（kWh）'][i] > 0:
                    data_stop.loc[i,'电量损失均值（kWh）'] = data_stop['电量损失1（kWh）'][i]
                if data_stop['电量损失1（kWh）'][i]>0 and data_stop['电量损失2（kWh）'][i]>0:
                    data_stop.loc[i,'电量损失均值（kWh）'] = round((data_stop['电量损失1（kWh）'][i] + data_stop['电量损失2（kWh）'][i])/2.,2) 
                                
    #Write result to csv file
    Path(args.resultfile).parent.mkdir(parents=True, exist_ok=True)
    data_stop[['site_id','device_id','iec_group_id','iec_level4_description','start_time','end_time','周围可用风机1','周围可用风机2','电量损失1（kWh）','电量损失2（kWh）','电量损失均值（kWh）']].to_csv(args.resultfile,index=False)
if __name__ == '__main__':
    main(sys.argv[1:])